

from apgl.util.Parameter import Parameter
import numpy

class  BarabasiAlbertGenerator():
    '''
    A class which generates graphs according to the Barabasi-Albert Model. We start with
    ell vertices, and then add a new vertex at each iteration with m edges. The probability
    of attachment to vertex i is d_i / \sum_j d_j where d_i is the degree of the ith vertex.
    '''

    def __init__(self, graph):
        """
        Create with an empty graph.

        :param graph: an intitial empty graph to populate with edges. 
        """
        self.graph = graph


    def generateGraph(self, ell, m):
        """
        Create a random graph according to the Barabasi-Albert Model

        :param ell: the initial number of vertices.
        :type ell: :class:`int`

        :param m: the number of edges to be added at each step
        :type m: :class:`int`
        """
        Parameter.checkInt(ell, 2, self.graph.getNumVertices())
        Parameter.checkInt(m, 0, ell)
        
        #Keep a list of node indices with degrees
        #First start off with ell vertices assume they each have degree 1, without
        #adding edges. This is a bit weird but seems the way to do it. 
        vertexList = range(0, ell)

        #Now perform preferential attachment, making sure we add m edges at each
        #iteration. 
        for i in range(ell, self.graph.getNumVertices()):
            perm = numpy.random.permutation(len(vertexList))
            numEdgesAdded = 0
            j = 0 

            while numEdgesAdded != m:
                ind = perm[j]
                vertexIndex = vertexList[ind]
                
                if self.graph.getEdge(i, vertexIndex) == None:
                    self.graph.addEdge(i, vertexIndex)
                    vertexList.append(i)
                    vertexList.append(vertexIndex)
                    numEdgesAdded = numEdgesAdded + 1 
                
                j = j+1


        return self.graph

    graph = None
