

from apgl.util.Parameter import Parameter 
import numpy

class ErdosRenyiGenerator():
    '''
    A class which generates graphs according to the Erdos-Renyi Model. At construction time, it
    takes an empty graph, for which it creates edges.
    '''
    def __init__(self, graph):
        self.graph = graph 
        
    def generateGraph(self, p, edgeWeight=1):
        '''
        Create an Erdos-Renyi graph with edge probability p and given edgeWeight.
        For all pairs of vertices in the graph, an edge exists with probability p.

        :param p: the probability of an edge
        :type p: :class:`float`

        :param edgeWeight: the non-zero weight of newly created edges.
        :type edgeWeight: :class:`float`
        '''
        Parameter.checkFloat(p, 0.0, 1.0)

        numVertices = self.graph.getNumVertices()
        undirected = self.graph.isUndirected()
        
        for i in range(0, numVertices): 
            v = numpy.random.rand(numVertices)
            edgeEnds = numpy.nonzero(v < p)[0]
            
            if undirected == True:
                for j in range(0, edgeEnds.shape[0]):
                    if i < edgeEnds[j]:
                        self.graph.addEdge(i, edgeEnds[j], edgeWeight)
            else:
                for j in range(0, edgeEnds.shape[0]):
                    if i != edgeEnds[j]:
                        self.graph.addEdge(i, edgeEnds[j], edgeWeight)
            
        return self.graph
            
    def clusteringCoefficient(self, p):
        '''
        Returns the clustering coefficient for the graph created with edge probability
        p. 
        '''
        Parameter.checkFloat(p, 0.0, 1.0)
        
        return p

    graph = None
    