"""
Some methods which are useful but didn't quite fit into the graph classes. 

"""
import logging 
import numpy 

class GraphUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def vertexLabelExamples(graph):
        """
        Return a set of examples with pairs of vertex labels connected by an
        edge. For undircted graphs there exists an example (v_i, v_j) for
        every (v_j, v_i). Also, there is a set of negative examples where the
        edge does not exist. 
        """
        numFeatures = graph.getVertexList().getNumFeatures()
        numEdges = graph.getNumEdges()

        #Also add non-edges
        logging.info("Computing graph complement")
        cGraph = graph.complement()
        logging.info("Done with " + str(cGraph.getNumEdges()) + " edges.")
        perm = numpy.random.permutation(cGraph.getNumEdges())[0:numEdges]

        X = graph.vertexLabelPairs(numpy.arange(numEdges))
        Xc = cGraph.vertexLabelPairs(perm)
        X = numpy.r_[X, Xc]

        y = numpy.ones(numEdges*2)
        y[numEdges:numEdges*2] = -1
        logging.debug(y)

        #If the graph is undirected add reverse edges
        if graph.isUndirected():
            X2 = numpy.zeros((numEdges*2, numFeatures*2))
            X2[:, 0:numFeatures] = X[:, numFeatures:numFeatures*2]
            X2[:, numFeatures:numFeatures*2] = X[:, 0:numFeatures]
            X = numpy.r_[X, X2]
            y = numpy.r_[y, y]

        return X, y

    @staticmethod 
    def treeDepth(treeGraph):
        #find root
        W = treeGraph.getSparseWeightMatrix()

        for i in range(treeGraph.getNumVertices()):
            if W[:, i].getnnz() == 0:
                root = i
                break

        distances = treeGraph.dijkstrasAlgorithm(root)
        return numpy.max(distances)