
import numpy
from apgl.util.Parameter import Parameter
from apgl.graph.VertexList import VertexList
from apgl.graph.SparseGraph import SparseGraph

class KroneckerGenerator():
    '''
    A class which generates graphs according to the Kronecker Epidemic method.
    '''
    def __init__(self, initialGraph, k, n):
        """
        Initialise with a starting graph, and number of iterations k and decay
        factor n. 
        """
        Parameter.checkInt(k, 1, float('inf'))
        Parameter.checkInt(n, 1, float('inf'))
        self.initialGraph = initialGraph
        self.k = k
        self.n = n 

    def setK(self, k):
        Parameter.checkInt(k, 1, float('inf'))
        self.k = k

    def generateGraph(self):
        """
        Generate a Kronecker graph
        """
        Wi = self.initialGraph.getWeightMatrix()

        if (numpy.diag(Wi)==numpy.zeros(Wi.shape[0])).any():
            raise ValueError("Initial graph must have all self-edges")

        W = Wi
        dW = numpy.diag(Wi)

        for i in range(1,self.k):
            W = numpy.kron(W, Wi)
            Wi = Wi/self.n

            #Keep the diagonal the same 
            Wi[numpy.diag_indices(Wi.shape[0])] = dW


        vList = VertexList(W.shape[0], 0)
        graph = SparseGraph(vList, self.initialGraph.isUndirected())
        graph.setWeightMatrix(W)

        return graph