from apgl.util import *
from apgl.graph import *
import unittest
import numpy 

class AbstractMatrixGraphTest(unittest.TestCase):
    def setUp(self):
        self.numVertices = 6
        self.numFeatures = 1
        self.vList = VertexList(self.numVertices, self.numFeatures)

        self.dGraph = DenseGraph(self.vList)
        self.dGraph.addEdge(0, 1, 1)
        self.dGraph.addEdge(1, 3, 1)
        self.dGraph.addEdge(0, 2, 2)
        self.dGraph.addEdge(2, 3, 5)
        self.dGraph.addEdge(0, 4, 1)
        self.dGraph.addEdge(3, 4, 1)

        self.dGraph2 = DenseGraph(self.vList, False)
        self.dGraph2.addEdge(0, 1, 1)
        self.dGraph2.addEdge(1, 3, 1)
        self.dGraph2.addEdge(0, 2, 2)
        self.dGraph2.addEdge(2, 3, 5)
        self.dGraph2.addEdge(0, 4, 1)
        self.dGraph2.addEdge(3, 4, 1)

    def testFloydWarshall(self):
        P = self.dGraph.floydWarshall()

        P2 = numpy.zeros((self.numVertices, self.numVertices))
        P2[0, :] = numpy.array([2, 1, 2, 2, 1, numpy.inf])
        P2[1, :] = numpy.array([1, 2, 3, 1, 2, numpy.inf])
        P2[2, :] = numpy.array([2, 3, 4, 4, 3, numpy.inf])
        P2[3, :] = numpy.array([2, 1, 4, 2, 1, numpy.inf])
        P2[4, :] = numpy.array([1, 2, 3, 1, 2, numpy.inf])
        P2[5, :] = numpy.array([numpy.inf, numpy.inf, numpy.inf, numpy.inf, numpy.inf, numpy.inf])
        
        self.assertTrue((P == P2).all())

        #Now test the directed graph
        P = self.dGraph2.floydWarshall()

        P2 = numpy.zeros((self.numVertices, self.numVertices))
        P2[0, :] = numpy.array([numpy.inf, 1, 2, 2, 1, numpy.inf])
        P2[1, :] = numpy.array([numpy.inf, numpy.inf, numpy.inf, 1, 2, numpy.inf])
        P2[2, :] = numpy.array([numpy.inf, numpy.inf, numpy.inf, 5, 6, numpy.inf])
        P2[3, :] = numpy.array([numpy.inf, numpy.inf, numpy.inf, numpy.inf, 1, numpy.inf])
        P2[4, :] = numpy.array([numpy.inf, numpy.inf, numpy.inf, numpy.inf, numpy.inf, numpy.inf])
        P2[5, :] = numpy.array([numpy.inf, numpy.inf, numpy.inf, numpy.inf, numpy.inf, numpy.inf])

        self.assertTrue((P == P2).all())

    #This needs to be done quite thoroughly 
    def testMaxProductPaths(self):
        numVertices = 6
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)

        dGraph = DenseGraph(vList)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 3, 0.1)
        dGraph.addEdge(0, 2, 0.2)
        dGraph.addEdge(2, 3, 0.5)
        dGraph.addEdge(0, 4, 0.1)
        dGraph.addEdge(3, 4, 0.1)

        P = dGraph.maxProductPaths()

        P2 = numpy.zeros((numVertices, numVertices))
        P2[0, :] = numpy.array([0.04, 0.1, 0.2, 0.1, 0.1, 0])
        P2[1, :] = numpy.array([0.1, 0.01, 0.05, 0.1, 0.01, 0])
        P2[2, :] = numpy.array([0.2, 0.05, 0.25, 0.5, 0.05, 0])
        P2[3, :] = numpy.array([0.1, 0.1, 0.5, 0.25, 0.1, 0])
        P2[4, :] = numpy.array([0.1, 0.01, 0.05, 0.1, 0.01, 0])
        P2[5, :] = numpy.array([0,0,0,0,0,0])

        self.assertAlmostEquals(numpy.linalg.norm(P - P2), 0, 6)

        #Now test on a directed graph
        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 3, 0.1)
        dGraph.addEdge(0, 2, 0.2)
        dGraph.addEdge(2, 3, 0.5)
        dGraph.addEdge(0, 4, 0.1)
        dGraph.addEdge(3, 4, 0.1)

        P = dGraph.maxProductPaths()

        P2 = numpy.zeros((numVertices, numVertices))
        P2[0, :] = numpy.array([0, 0.1, 0.2, 0.1, 0.1, 0])
        P2[1, :] = numpy.array([0, 0, 0, 0.1, 0.01, 0])
        P2[2, :] = numpy.array([0, 0, 0, 0.5, 0.05, 0])
        P2[3, :] = numpy.array([0, 0, 0, 0, 0.1, 0])
        P2[4, :] = numpy.array([0,0,0,0,0,0])
        P2[5, :] = numpy.array([0,0,0,0,0,0])

        self.assertAlmostEquals(numpy.linalg.norm(P - P2), 0, 6)

    def testMaybeIsomorphicWith(self):
        numVertices = 6
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)

        dGraph = DenseGraph(vList, True)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 3, 0.1)
        dGraph.addEdge(0, 2, 0.2)
        dGraph.addEdge(2, 3, 0.5)
        dGraph.addEdge(0, 4, 0.1)
        dGraph.addEdge(3, 4, 0.1)
        
        dGraph2 = DenseGraph(vList, True)
        dGraph2.addEdge(0, 1, 0.1)
        dGraph2.addEdge(1, 3, 0.1)
        dGraph2.addEdge(0, 2, 0.2)
        dGraph2.addEdge(2, 3, 0.5)
        dGraph2.addEdge(0, 4, 0.1)
        dGraph2.addEdge(3, 4, 0.1)
        dGraph2.addEdge(4, 5, 0.1)

        dGraph3 = DenseGraph(vList, True)
        dGraph3.addEdge(2, 4, 0.1)
        dGraph3.addEdge(4, 5, 0.1)
        dGraph3.addEdge(2, 1, 0.2)
        dGraph3.addEdge(1, 5, 0.5)
        dGraph3.addEdge(2, 0, 0.1)
        dGraph3.addEdge(5, 0, 0.1)

        self.assertTrue(dGraph.maybeIsomorphicWith(dGraph))
        self.assertFalse(dGraph.maybeIsomorphicWith(dGraph2))
        self.assertTrue(dGraph.maybeIsomorphicWith(dGraph3))

    def testVertexLabelPairs(self):
        numVertices = 6
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.array([numpy.arange(0, 6)]).T)

        dGraph = DenseGraph(vList, True)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 3, 0.1)
        dGraph.addEdge(0, 2, 0.2)
        dGraph.addEdge(2, 3, 0.5)
        dGraph.addEdge(0, 4, 0.1)
        dGraph.addEdge(3, 4, 0.1)

        tol = 10**-6
        edges = dGraph.getAllEdges()

        X = dGraph.vertexLabelPairs(edges)
        self.assertTrue(numpy.linalg.norm(X - edges) < tol )


        X = dGraph.vertexLabelPairs(edges[[5, 2, 1], :])
        self.assertTrue(numpy.linalg.norm(X - edges[[5,2,1], :]) < tol )

        #Try a bigger graph
        numVertices = 6
        numFeatures = 2
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.randn(numVertices, numFeatures))

        dGraph = DenseGraph(vList, True)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 3, 0.1)

        edges = dGraph.getAllEdges()

        X = dGraph.vertexLabelPairs(edges)
        self.assertTrue(numpy.linalg.norm(X[0, 0:numFeatures] - vList.getVertex(1)) < tol )
        self.assertTrue(numpy.linalg.norm(X[0, numFeatures:numFeatures*2] - vList.getVertex(0)) < tol )
        self.assertTrue(numpy.linalg.norm(X[1, 0:numFeatures] - vList.getVertex(3)) < tol )
        self.assertTrue(numpy.linalg.norm(X[1, numFeatures:numFeatures*2] - vList.getVertex(1)) < tol )

        #Try directed graphs
        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 3, 0.1)

        edges = dGraph.getAllEdges()

        X = dGraph.vertexLabelPairs(edges)
        self.assertTrue(numpy.linalg.norm(X[0, 0:numFeatures] - vList.getVertex(0)) < tol )
        self.assertTrue(numpy.linalg.norm(X[0, numFeatures:numFeatures*2] - vList.getVertex(1)) < tol )
        self.assertTrue(numpy.linalg.norm(X[1, 0:numFeatures] - vList.getVertex(1)) < tol )
        self.assertTrue(numpy.linalg.norm(X[1, numFeatures:numFeatures*2] - vList.getVertex(3)) < tol )

    def testDiameter(self):
        numVertices = 10
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)


        dGraph = DenseGraph(vList, True)
        dGraph.addEdge(0, 1)
        dGraph.addEdge(1, 2)
        dGraph.addEdge(1, 3)

        self.assertEquals(dGraph.diameter(), 2)

        dGraph.addEdge(3, 2)

        self.assertEquals(dGraph.diameter(), 2)

        dGraph.addEdge(3, 4)

        self.assertEquals(dGraph.diameter(), 3)

        dGraph.addEdge(4, 5)

        self.assertEquals(dGraph.diameter(), 4)

        dGraph.addEdge(0, 5)

        self.assertEquals(dGraph.diameter(), 3)


        #Now try directed graphs
        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1)
        dGraph.addEdge(1, 2)
        dGraph.addEdge(1, 3)

        self.assertEquals(dGraph.diameter(), 2)

        dGraph.addEdge(4, 3)

        self.assertEquals(dGraph.diameter(), 2)

        dGraph.addEdge(5, 4)
        dGraph.addEdge(6, 5)

        self.assertEquals(dGraph.diameter(), 3)


        dGraph.addEdge(6, 6)
        self.assertEquals(dGraph.diameter(), 3)

        #Test on graph with no edges

        dGraph = DenseGraph(vList, False)
        self.assertEquals(dGraph.diameter(), 0)

    def testGetEdges(self):
        numVertices = 10
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)

        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 2, 0.2)
        dGraph.addEdge(1, 3, 0.3)

        edges = dGraph.getAllEdges()


        self.assertTrue((dGraph.getEdgeValues(edges) == numpy.array([0.1, 0.2, 0.3])).all())

    def testSave(self):
        numVertices = 10
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.rand(numVertices, numFeatures))

        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 2, 0.2)
        dGraph.addEdge(1, 3, 0.3)

        tempDir = PathDefaults.getTempDir()

        dGraph.save(tempDir + "testGraph.dat")

    def testLoad(self):
        numVertices = 10
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.rand(numVertices, numFeatures))

        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 2, 0.2)
        dGraph.addEdge(1, 3, 0.3)

        tempDir = PathDefaults.getTempDir()
        tempFile = tempDir + "testGraph.dat"

        dGraph.save(tempFile)

        dGraph2 = DenseGraph.load(tempFile)

        self.assertEquals(dGraph.getNumVertices(), dGraph.getNumVertices())
        self.assertEquals(dGraph.getNumEdges(), dGraph.getNumEdges())
        self.assertTrue(dGraph2.isUndirected() == False) 
        self.assertTrue((dGraph.getVertexList().getVertices(range(numVertices)) == dGraph2.getVertexList().getVertices(range(numVertices))).all())
        self.assertTrue((dGraph.getAllEdges() == dGraph2.getAllEdges()).all())
        self.assertTrue(dGraph2.getEdge(0, 1) == 0.1)
        self.assertTrue(dGraph2.getEdge(1, 2) == 0.2)
        self.assertTrue(dGraph2.getEdge(1, 3) == 0.3)

        #Now do same tests for sparseGraph
        sGraph = SparseGraph(vList, True)
        sGraph.addEdge(0, 1, 0.1)
        sGraph.addEdge(1, 2, 0.2)
        sGraph.addEdge(1, 3, 0.3)

        tempDir = PathDefaults.getTempDir()
        tempFile = tempDir + "testGraph.dat"

        sGraph.save(tempFile)

        sGraph2 = SparseGraph.load(tempFile)

        self.assertEquals(sGraph.getNumVertices(), sGraph.getNumVertices())
        self.assertEquals(sGraph.getNumEdges(), sGraph.getNumEdges())
        self.assertTrue(sGraph2.isUndirected() == True)
        self.assertTrue((sGraph.getVertexList().getVertices(range(numVertices)) == sGraph2.getVertexList().getVertices(range(numVertices))).all())
        self.assertTrue((sGraph.getAllEdges() == sGraph2.getAllEdges()).all())
        self.assertTrue(sGraph2.getEdge(0, 1) == 0.1)
        self.assertTrue(sGraph2.getEdge(1, 2) == 0.2)
        self.assertTrue(sGraph2.getEdge(1, 3) == 0.3)

    def testSetVertexList(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.rand(numVertices, numFeatures))

        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 0.1)
        dGraph.addEdge(1, 2, 0.2)

        self.assertTrue((dGraph.getVertex(0) == vList.getVertex(0)).all())
        self.assertTrue((dGraph.getVertex(1) == vList.getVertex(1)).all())
        self.assertTrue((dGraph.getVertex(2) == vList.getVertex(2)).all())

        vList2 = VertexList(numVertices, numFeatures+2)
        vList2.setVertices(numpy.random.rand(numVertices, numFeatures+2))

        dGraph.setVertexList(vList2)
        self.assertTrue((dGraph.getVertex(0) == vList2.getVertex(0)).all())
        self.assertTrue((dGraph.getVertex(1) == vList2.getVertex(1)).all())
        self.assertTrue((dGraph.getVertex(2) == vList2.getVertex(2)).all())

        vList3 = VertexList(numVertices+1, numFeatures)
        self.assertRaises(ValueError, dGraph.setVertexList, vList3)

    def testAddEdges(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.rand(numVertices, numFeatures))
        dGraph = DenseGraph(vList)

        edgeIndexArray = numpy.array([[1,2], [2,3]])

        dGraph.addEdges(edgeIndexArray)
        self.assertEquals(dGraph.getEdge(1, 2), 1)
        self.assertEquals(dGraph.getEdge(3, 2), 1)
        self.assertEquals(dGraph.getEdge(2, 3), 1)
        self.assertEquals(dGraph.getEdge(2, 1), 1)
        self.assertEquals(dGraph.getNumEdges(), 2)

        dGraph = DenseGraph(vList, False)
        dGraph.addEdges(edgeIndexArray)

        self.assertEquals(dGraph.getNumEdges(), 2)
        self.assertEquals(dGraph.getEdge(1, 2), 1)
        self.assertEquals(dGraph.getEdge(2, 3), 1)

        edgeValues = numpy.array([0.1, 0.2])
        dGraph.addEdges(edgeIndexArray, edgeValues)
        self.assertEquals(dGraph.getEdge(1, 2), 0.1)
        self.assertEquals(dGraph.getEdge(2, 3), 0.2)

        dGraph = DenseGraph(vList)
        dGraph.addEdges(edgeIndexArray, edgeValues)
        self.assertEquals(dGraph.getEdge(1, 2), 0.1)
        self.assertEquals(dGraph.getEdge(2, 3), 0.2)
        self.assertEquals(dGraph.getEdge(2, 1), 0.1)
        self.assertEquals(dGraph.getEdge(3, 2), 0.2)

        edgeValues = numpy.array([0.1, 0.0])
        self.assertRaises(ValueError, dGraph.addEdges, edgeIndexArray, edgeValues) 

    def testSetWeightMatrix(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.rand(numVertices, numFeatures))
        dGraph = DenseGraph(vList)

        dGraph.addEdge(0, 1)
        dGraph.addEdge(0, 2)

        W = numpy.zeros((numVertices, numVertices))
        W[1, 1] = 1
        W[2, 1] = 1
        W[1, 2] = 1

        dGraph.setWeightMatrix(W)
        self.assertTrue((dGraph.getAllEdges() == numpy.array([[1, 1], [2, 1]])).all())

        W[1, 3] = 1
        self.assertRaises(ValueError, dGraph.setWeightMatrix, W)
        W = numpy.zeros((numVertices, numVertices+1))
        self.assertRaises(ValueError, dGraph.setWeightMatrix, W)

        #Now, see if it works for undirected graphs
        dGraph = DenseGraph(vList, False)
        W = numpy.zeros((numVertices, numVertices))
        W[1, 0] = 1
        W[3, 1] = 1
        W[1, 3] = 1

        dGraph.setWeightMatrix(W)
        self.assertTrue((dGraph.getAllEdges() == numpy.array([[1, 0], [1,3], [3, 1]])).all())

    def testEffectiveDiameter(self):
        numVertices = 10
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)


        dGraph = DenseGraph(vList, True)
        dGraph.addEdge(1, 4)
        dGraph.addEdge(0, 1)
        dGraph.addEdge(1, 2)
        dGraph.addEdge(1, 3)

        self.assertEquals(dGraph.diameter(), 2)
        self.assertEquals(dGraph.effectiveDiameter(1.0), 2)
        self.assertEquals(dGraph.effectiveDiameter(0.5), 2)

        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1)
        dGraph.addEdge(2, 3)
        dGraph.addEdge(4, 5)
        dGraph.addEdge(5, 6)
        dGraph.addEdge(7, 8)
        dGraph.addEdge(8, 9)

        self.assertEquals(dGraph.effectiveDiameter(1.0), 2)
        self.assertEquals(dGraph.effectiveDiameter(0.75), 1)
        self.assertEquals(dGraph.effectiveDiameter(0.5), 1)

if __name__ == '__main__':
    unittest.main()
