
from apgl.graph import * 
import unittest
import numpy

#TODO: Test undirected graphs

class DenseGraphTest(unittest.TestCase):
    def setUp(self):
        self.numVertices = 10
        self.numFeatures = 3
        self.vList = VertexList(self.numVertices, self.numFeatures)

        self.dGraph = DenseGraph(self.vList)

    def testAddEdge(self):
        self.dGraph.addEdge(1, 5, 2)

        self.assertEquals(self.dGraph.getEdge(1,5), 2)
        self.assertEquals(self.dGraph.getEdge(5,1), 2)

        self.assertEquals(self.dGraph.getEdge(2,5), None)

        self.assertRaises(ValueError, self.dGraph.addEdge, 1, 3, 0)

    def testRemoveEdge(self):
        self.dGraph.addEdge(1, 5, 2)
        
        self.assertEquals(self.dGraph.getEdge(1,5), 2)
        self.assertEquals(self.dGraph.getEdge(5,1), 2)
        
        self.dGraph.removeEdge(1,5)
        
        self.assertEquals(self.dGraph.getEdge(1,5), None)
        self.assertEquals(self.dGraph.getEdge(5,2), None)

    def testGetNeighbours(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        dGraph = DenseGraph(vList)

        dGraph.addEdge(1, 5, 2)
        dGraph.addEdge(1, 3, 5)
        dGraph.addEdge(1, 9, 1)
        dGraph.addEdge(2, 3, 2)

        self.assertTrue((dGraph.neighbours(1) == numpy.array([3,5,9])).all())
        self.assertTrue((dGraph.neighbours(2) == numpy.array([3])).all())
        self.assertTrue((dGraph.neighbours(3) == numpy.array([1,2])).all())
        self.assertTrue((dGraph.neighbours(4) == numpy.array([])).all())

    def testGetAllEdges(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)

        dGraph.addEdge(0, 1, 5)
        dGraph.addEdge(1, 2, 2)
        dGraph.addEdge(2, 3, 2)
        dGraph.addEdge(2, 2, 2)

        edges = dGraph.getAllEdges()

        self.assertEquals(edges.shape[0], 4)
        self.assertTrue((edges[0, :]== numpy.array([1,0])).all())
        self.assertTrue((edges[1, :]== numpy.array([2,1])).all())
        self.assertTrue((edges[2, :]== numpy.array([2,2])).all())
        self.assertTrue((edges[3, :]== numpy.array([3,2])).all())


        #Test a directed graph
        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 5)
        dGraph.addEdge(1, 2, 2)
        dGraph.addEdge(2, 3, 2)
        dGraph.addEdge(2, 2, 2)
        dGraph.addEdge(2, 1, 2)

        edges = dGraph.getAllEdges()

        self.assertEquals(edges.shape[0], 5)
        self.assertTrue((edges[0, :]== numpy.array([0,1])).all())
        self.assertTrue((edges[1, :]== numpy.array([1,2])).all())
        self.assertTrue((edges[2, :]== numpy.array([2,1])).all())
        self.assertTrue((edges[3, :]== numpy.array([2,2])).all())
        self.assertTrue((edges[4, :]== numpy.array([2,3])).all())

        #Test graph with no edges
        dGraph = DenseGraph(vList)
        edges = dGraph.getAllEdges()
        self.assertEquals(edges.shape, (0, 2))

    def testGetNumEdges(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        dGraph = DenseGraph(vList)
        dGraph.addEdge(0, 1, 1)

        self.assertEquals(dGraph.getNumEdges(), 1)

        dGraph.addEdge(3, 4, 1)
        dGraph.addEdge(3, 4, 1)

        self.assertEquals(dGraph.getNumEdges(), 2)

        dGraph.addEdge(5, 5, 1)
        self.assertEquals(dGraph.getNumEdges(), 3)

        dGraph.addEdge(8, 8, 1)
        dGraph.addEdge(8, 8, 1)
        self.assertEquals(dGraph.getNumEdges(), 4)

        #Now test directed graphs
        dGraph = DenseGraph(vList, False)
        dGraph.addEdge(0, 1, 1)

        self.assertEquals(dGraph.getNumEdges(), 1)

        dGraph.addEdge(3, 4, 1)
        dGraph.addEdge(3, 4, 1)

        self.assertEquals(dGraph.getNumEdges(), 2)

        dGraph.addEdge(5, 5, 1)
        self.assertEquals(dGraph.getNumEdges(), 3)

        dGraph.addEdge(8, 8, 1)
        dGraph.addEdge(8, 8, 1)
        self.assertEquals(dGraph.getNumEdges(), 4)

    def testGetNumVertices(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)

        self.assertEquals(dGraph.getNumVertices(), numVertices)

        numVertices = 0
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)

        self.assertEquals(dGraph.getNumVertices(), numVertices)

    def testGetEdge(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)
        dGraph.addEdge(2, 5, 1)
        dGraph.addEdge(4, 8, 34)

        self.assertEquals(dGraph.getEdge(2, 5), 1)
        self.assertEquals(dGraph.getEdge(5, 2), 1)
        self.assertEquals(dGraph.getEdge(4, 8), 34)
        self.assertEquals(dGraph.getEdge(8, 4), 34)

        self.assertEquals(dGraph.getEdge(4, 4), None)

    def testGetVertex(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)
        dGraph.setVertex(1, numpy.array([4, 5, 2]))

        self.assertRaises(ValueError, dGraph.setVertex, -1, numpy.array([4, 5, 2]))
        self.assertRaises(ValueError, dGraph.setVertex, 11, numpy.array([4, 5, 2]))
        self.assertRaises(ValueError, dGraph.setVertex, 2, numpy.array([4, 5, 2, 8]))
        self.assertRaises(ValueError, dGraph.setVertex, 2, numpy.array([4, 5]))
        self.assertTrue((dGraph.getVertex(1) == numpy.array([4, 5, 2])).all())
        self.assertTrue((dGraph.getVertex(0) == numpy.array([0, 0, 0])).all())

    def testSetVertex(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)
        dGraph.setVertex(1, numpy.array([4, 5, 2]))

        self.assertTrue((dGraph.getVertex(1) == numpy.array([4, 5, 2])).all())
        self.assertTrue((dGraph.getVertex(0) == numpy.array([0, 0, 0])).all())

        dGraph.setVertex(1, numpy.array([8, 3, 1]))
        self.assertTrue((dGraph.getVertex(1) == numpy.array([8, 3, 1])).all())

    def testIsUndirected(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList, True)
        self.assertEquals(dGraph.isUndirected(), True)

        dGraph = DenseGraph(vList, False)
        self.assertEquals(dGraph.isUndirected(), False)

    def testGetAllVertexIds(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)

        self.assertTrue((dGraph.getAllVertexIds() == numpy.array(range(0, numVertices))).all())

        numVertices = 0
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)

        self.assertTrue((dGraph.getAllVertexIds() == numpy.array(range(0, numVertices))).all())
    
    def testGetNumVertexFeatures(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        dGraph = DenseGraph(vList)

        self.assertEquals(dGraph.getNumVertexFeatures(), numFeatures)

    def testStr(self):
        print(self.dGraph)

    def testRemoveAllEdges(self):
        numVertices = 6
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        dGraph = DenseGraph(vList, True)
        dGraph.addEdge(0, 1)
        dGraph.addEdge(0, 2)
        dGraph.addEdge(0, 3)
        dGraph.addEdge(2, 1)
        dGraph.addEdge(2, 3)
        dGraph.addEdge(4, 1)

        dGraph.removeAllEdges()
        self.assertTrue(dGraph.getEdge(0,1) == None)

    def testGetAdjacencyMatrix(self):
        numVertices = 3
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)

        graph = DenseGraph(vList, False)
        graph.addEdge(0, 1, 0.5)
        graph.addEdge(2, 1, 0.2)
        graph.addEdge(1, 1, 0.1)

        A = graph.adjacencyMatrix()
        W = graph.getWeightMatrix()


        W2 = numpy.zeros((numVertices, numVertices))
        A2 = numpy.zeros((numVertices, numVertices))

        W2[0,1]= 0.5
        W2[2,1]= 0.2
        W2[1,1]= 0.1

        A2[0,1]= 1
        A2[2,1]= 1
        A2[1,1]= 1

        print(W)
        print(A)

        self.assertTrue((W == W2).all())
        self.assertTrue((A == A2).all())

    def testComplement(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = DenseGraph(vList)

        graph.addEdge(0, 1, 0.1)
        graph.addEdge(2, 1, 0.2)
        graph.addEdge(4, 2, 0.5)
        graph.addEdge(6, 7, 0.9)
        graph.addEdge(3, 3, 1.1)

        graph2 = graph.complement()

        self.assertTrue(graph2.isUndirected())
        self.assertEquals(graph2.getEdge(0, 1), None)
        self.assertEquals(graph2.getEdge(2, 1), None)
        self.assertEquals(graph2.getEdge(4, 2), None)
        self.assertEquals(graph2.getEdge(6, 7), None)
        self.assertEquals(graph2.getEdge(3, 3), None)

        self.assertEquals(graph2.getEdge(0,0), 1)
        self.assertEquals(graph2.getNumEdges(), 50)

        #Now test on directed graphs
        vList = VertexList(numVertices, numFeatures)
        graph = DenseGraph(vList, False)

        graph.addEdge(0, 1, 0.1)
        graph.addEdge(2, 1, 0.2)
        graph.addEdge(4, 2, 0.5)
        graph.addEdge(6, 7, 0.9)
        graph.addEdge(3, 3, 1.1)

        graph2 = graph.complement()

        self.assertFalse(graph2.isUndirected())
        self.assertEquals(graph2.getEdge(0, 1), None)
        self.assertEquals(graph2.getEdge(2, 1), None)
        self.assertEquals(graph2.getEdge(4, 2), None)
        self.assertEquals(graph2.getEdge(6, 7), None)
        self.assertEquals(graph2.getEdge(3, 3), None)

        self.assertEquals(graph2.getEdge(0,0), 1)
        self.assertEquals(graph2.getEdge(1,0), 1)
        self.assertEquals(graph2.getNumEdges(), 95)

    def testGetNumDirEdges(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = DenseGraph(vList)
        graph.addEdge(0, 1, 0.1)
        graph.addEdge(1, 2, 0.1)

        self.assertTrue(graph.getNumDirEdges() == 4)
        graph.addEdge(1, 1)
        self.assertTrue(graph.getNumDirEdges() == 5)

        graph = DenseGraph(vList, False)
        graph.addEdge(0, 1)
        graph.addEdge(1, 2)

        self.assertTrue(graph.getNumDirEdges() == 2)
        graph.addEdge(1, 1)
        self.assertTrue(graph.getNumDirEdges() == 3)

    def testGeneralVertexList(self):
        #Very brief test to make sure sparse graph works with general vertex lists
        numVertices = 10
        vList = GeneralVertexList(numVertices)
        vList.setVertex(0, "a")
        vList.setVertex(1, "b")
        vList.setVertex(5, "c")

        graph = DenseGraph(vList)
        graph.addEdge(0, 1)
        graph.addEdge(1, 2)
        graph.addEdge(1, 5)
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()