'''
Created on 3 Jul 2009

@author: charanpal
'''
from apgl.graph.SparseGraph import SparseGraph
from apgl.graph.VertexList import VertexList
from apgl.graph.ErdosRenyiGenerator import ErdosRenyiGenerator
import unittest

class ErdoRenyiGeneratorTest(unittest.TestCase):
    def setUp(self):    
        self.numVertices = 10; 
        self.numFeatures = 2; 
        
        self.vList = VertexList(self.numVertices, self.numFeatures)
        self.sGraph = SparseGraph(self.vList) 
        self.erg = ErdosRenyiGenerator(self.sGraph)

        
    def testGenerateGraph(self):
        edgeWeight = 2.5
        
        #undirected = True 
        p = 0.0
             
        self.sGraph.removeAllEdges()
        sGraph = self.erg.generateGraph(p, edgeWeight)
        self.assertEquals(sGraph.getNumEdges(), 0)
        
        undirected = False
        self.sGraph = SparseGraph(self.vList, undirected) 
        self.sGraph.removeAllEdges()
        self.erg = ErdosRenyiGenerator(self.sGraph)
        sGraph = self.erg.generateGraph(p, edgeWeight)
        self.assertEquals(sGraph.getNumEdges(), 0)
        
        p = 1.0
        undirected = True
        self.sGraph = SparseGraph(self.vList, undirected) 
        self.sGraph.removeAllEdges()
        self.erg = ErdosRenyiGenerator(self.sGraph)
        sGraph = self.erg.generateGraph(p, edgeWeight)
        self.assertEquals(sGraph.getNumEdges(), (self.numVertices*self.numVertices-self.numVertices)/2)
        
        p = 1.0
        undirected = False
        self.sGraph = SparseGraph(self.vList, undirected) 
        self.sGraph.removeAllEdges()
        self.erg = ErdosRenyiGenerator(self.sGraph)
        sGraph = self.erg.generateGraph(p, edgeWeight)
        self.assertEquals(sGraph.getNumEdges(), self.numVertices*self.numVertices-self.numVertices)
        
        self.assertEquals(sGraph.getEdge(1, 2), edgeWeight)
        self.assertEquals(sGraph.getEdge(1, 1), None)

        p = 0.5
        undirected = False
        self.sGraph = SparseGraph(self.vList, undirected)
        self.sGraph.removeAllEdges()
        self.erg = ErdosRenyiGenerator(self.sGraph)
        sGraph = self.erg.generateGraph(p, edgeWeight)

        print(sGraph.getNumEdges())
    
    def tearDown(self):
        pass

    def testInit(self):
        pass     
    
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(ErdoRenyiGeneratorTest)
    unittest.TextTestRunner(verbosity=2).run(suite)