'''
Created on 1 Jul 2009

@author: charanpal
'''
from apgl.graph import * 
import unittest
import numpy

#TODO: Test undirected graphs 

class SparseGraphTest(unittest.TestCase):
    def setUp(self):
        self.numVertices = 10
        self.numFeatures = 3
        self.vList = VertexList(self.numVertices, self.numFeatures)
        
        self.sGraph = SparseGraph(self.vList)
        
        pass
        
    def tearDown(self):
        pass

    def testInit(self):
        numVertices = 0
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        self.assertRaises(ValueError, SparseGraph, vList)

    def testAddEdge(self):
        self.sGraph.addEdge(1, 5, 2)
        
        self.assertEquals(self.sGraph.getEdge(1,5), 2)
        self.assertEquals(self.sGraph.getEdge(5,1), 2)
        
        self.assertEquals(self.sGraph.getEdge(2,5), None)
        
        self.assertRaises(ValueError, self.sGraph.addEdge, 1, 3, 0)
        
    def testRemoveEdge(self):
        self.sGraph.addEdge(1, 5, 2)
        
        self.assertEquals(self.sGraph.getEdge(1,5), 2)
        self.assertEquals(self.sGraph.getEdge(5,1), 2)
        
        self.sGraph.removeEdge(1,5)
        
        self.assertEquals(self.sGraph.getEdge(1,5), None)
        self.assertEquals(self.sGraph.getEdge(5,2), None)
        
    def testGetNeighbours(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(1, 5, 2)
        sGraph.addEdge(1, 3, 5)
        sGraph.addEdge(1, 9, 1)
        sGraph.addEdge(2, 3, 2)
        
        self.assertTrue((sGraph.neighbours(1) == numpy.array([3,5,9])).all())
        self.assertTrue((sGraph.neighbours(2) == numpy.array([3])).all())
        self.assertTrue((sGraph.neighbours(3) == numpy.array([1,2])).all())
        self.assertTrue((sGraph.neighbours(4) == numpy.array([])).all())

        #Test this function for directed graphs
        sGraph = SparseGraph(vList, False)

        sGraph.addEdge(1, 5, 2)
        sGraph.addEdge(1, 3, 5)
        sGraph.addEdge(9, 1, 1)
        sGraph.addEdge(2, 3, 2)

        self.assertTrue((sGraph.neighbours(1) == numpy.array([3,5])).all())
        self.assertTrue((sGraph.neighbours(2) == numpy.array([3])).all())
        self.assertTrue((sGraph.neighbours(3) == numpy.array([])).all())
        self.assertTrue((sGraph.neighbours(9) == numpy.array([1])).all())



    def testGetClusteringCoefficient(self):
        numVertices = 3
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        
        #1st graph - take 3 nodes in a line
        sGraph = SparseGraph(vList)
        sGraph.addEdge(0, 1, 2)
        sGraph.addEdge(1, 2, 5)
        
        self.assertEqual(sGraph.clusteringCoefficient(), 0)
        
        #Now, form a triangle 
        sGraph.addEdge(0, 2, 5)
        self.assertEqual(sGraph.clusteringCoefficient(), 1)
        
        #2nd Graph - taken from Newman 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        sGraph.addEdge(0, 1, 2)
        sGraph.addEdge(0, 2, 2)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(2, 3, 2)
        sGraph.addEdge(2, 4, 2)
        
        self.assertEqual(sGraph.clusteringCoefficient(), float(3)/8)
        
        #3rd graph - has no edges 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        self.assertEqual(sGraph.clusteringCoefficient(), 0.0)
        
    def testGetDegreeDistribution(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        sGraph.addEdge(0, 1, 2)
        sGraph.addEdge(0, 2, 2)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(2, 3, 2)
        sGraph.addEdge(2, 4, 2)
        
        self.assertTrue((sGraph.degreeDistribution() == numpy.array([0, 2, 2, 0, 1])).all())
        
        #Try empty graph 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        self.assertTrue((sGraph.degreeDistribution() == numpy.array([5])).all())
        
        #Try a star like graph 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        sGraph.addEdge(0, 0, 2)
        sGraph.addEdge(0, 1, 2)
        sGraph.addEdge(0, 2, 2)
        sGraph.addEdge(0, 3, 2)
        sGraph.addEdge(0, 4, 2)
         
        self.assertTrue((sGraph.degreeDistribution() == numpy.array([0, 4, 0, 0, 0, 1])).all())

        #Test obtaining a subgraph and then the degree distribution
        subGraph = sGraph.subgraph([0,1,2,3])
        print(subGraph.degreeDistribution())


    def testDijkstrasAlgorithm(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 5)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(1, 3, 2)
        sGraph.addEdge(2, 4, 2)
        
        self.assertTrue((sGraph.dijkstrasAlgorithm(0) == numpy.array([0, 1, 2, 2, 3])).all())
        self.assertTrue((sGraph.dijkstrasAlgorithm(1) == numpy.array([1, 0, 1, 1, 2])).all())
        self.assertTrue((sGraph.dijkstrasAlgorithm(2) == numpy.array([2, 1, 0, 2, 1])).all())
        self.assertTrue((sGraph.dijkstrasAlgorithm(3) == numpy.array([2, 1, 2, 0, 3])).all())
        self.assertTrue((sGraph.dijkstrasAlgorithm(4) == numpy.array([3, 2, 1, 3, 0])).all())
        
        #Test a graph which has an isolated node 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 5)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(1, 3, 2)
        
        self.assertTrue((sGraph.dijkstrasAlgorithm(0) == numpy.array([0, 1, 2, 2, numpy.inf])).all())

        #Test a graph in a ring
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 5)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(2, 3, 2)
        sGraph.addEdge(3, 4, 2)
        sGraph.addEdge(4, 0, 2)
        
        self.assertTrue((sGraph.dijkstrasAlgorithm(0) == numpy.array([0, 1, 2, 2, 1])).all())
        
    def testGetGeodesicDistance(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 5)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(2, 3, 2)
        sGraph.addEdge(3, 4, 2)
        sGraph.addEdge(4, 0, 2)
        
        self.assertEquals(sGraph.geodesicDistance(), 37/15.0)
        
        #Test a string of vertices 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 1)
        sGraph.addEdge(1, 2, 1)
        sGraph.addEdge(2, 3, 1)
        sGraph.addEdge(3, 4, 1)
        
        self.assertEquals(sGraph.geodesicDistance(), 4.0/3)
        
        #Test case with isolated node 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 1)
        sGraph.addEdge(1, 2, 1)
        sGraph.addEdge(2, 3, 1)
        
        self.assertEquals(sGraph.geodesicDistance(), 2.0/3)
    
    def testGetHarmonicGeodesicDistance(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 1)
        sGraph.addEdge(1, 2, 1)
        sGraph.addEdge(2, 3, 1)
        sGraph.addEdge(3, 4, 1)
        sGraph.addEdge(4, 0, 1)
        
        self.assertEquals(sGraph.harmonicGeodesicDistance(), 1.0/2)
        
        #Test a string of vertices 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 1)
        sGraph.addEdge(1, 2, 1)
        sGraph.addEdge(2, 3, 1)
        sGraph.addEdge(3, 4, 1)
        
        self.assertAlmostEquals(sGraph.harmonicGeodesicDistance(), 77.0/180, 5)
        
        #Test case with isolated node 
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 1)
        sGraph.addEdge(1, 2, 1)
        sGraph.addEdge(2, 3, 1)
        
        self.assertAlmostEquals(sGraph.harmonicGeodesicDistance(), 13.0/45, 5)
        
    def testGetAllEdges(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        
        sGraph.addEdge(0, 1, 5)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(2, 3, 2)
        sGraph.addEdge(2, 2, 2)
        
        edges = sGraph.getAllEdges()
        
        self.assertEquals(edges.shape[0], 4)
        self.assertTrue((edges[0, :]== numpy.array([1,0])).all())
        self.assertTrue((edges[1, :]== numpy.array([2,1])).all())
        self.assertTrue((edges[2, :]== numpy.array([2,2])).all())
        self.assertTrue((edges[3, :]== numpy.array([3,2])).all())
        
        
        #Test a directed graph 
        sGraph = SparseGraph(vList, False)
        sGraph.addEdge(0, 1, 5)
        sGraph.addEdge(1, 2, 2)
        sGraph.addEdge(2, 3, 2)
        sGraph.addEdge(2, 2, 2)
        sGraph.addEdge(2, 1, 2)
        
        edges = sGraph.getAllEdges()
        
        self.assertEquals(edges.shape[0], 5)
        self.assertTrue((edges[0, :]== numpy.array([0,1])).all())
        self.assertTrue((edges[1, :]== numpy.array([1,2])).all())
        self.assertTrue((edges[2, :]== numpy.array([2,1])).all())
        self.assertTrue((edges[3, :]== numpy.array([2,2])).all())
        self.assertTrue((edges[4, :]== numpy.array([2,3])).all())

        #Test graph with no edges
        sGraph = SparseGraph(vList)
        edges = sGraph.getAllEdges()
        self.assertEquals(edges.shape, (0, 2))

    def testGetNumEdges(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        sGraph = SparseGraph(vList)
        sGraph.addEdge(0, 1, 1)

        self.assertEquals(sGraph.getNumEdges(), 1)

        sGraph.addEdge(3, 4, 1)
        sGraph.addEdge(3, 4, 1)

        self.assertEquals(sGraph.getNumEdges(), 2)

        sGraph.addEdge(5, 5, 1)
        self.assertEquals(sGraph.getNumEdges(), 3)

        sGraph.addEdge(8, 8, 1)
        sGraph.addEdge(8, 8, 1)
        self.assertEquals(sGraph.getNumEdges(), 4)

        #Now test directed graphs
        sGraph = SparseGraph(vList, False)
        sGraph.addEdge(0, 1, 1)

        self.assertEquals(sGraph.getNumEdges(), 1)

        sGraph.addEdge(3, 4, 1)
        sGraph.addEdge(3, 4, 1)

        self.assertEquals(sGraph.getNumEdges(), 2)

        sGraph.addEdge(5, 5, 1)
        self.assertEquals(sGraph.getNumEdges(), 3)

        sGraph.addEdge(8, 8, 1)
        sGraph.addEdge(8, 8, 1)
        self.assertEquals(sGraph.getNumEdges(), 4)

    def testGetNumVertices(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)

        self.assertEquals(sGraph.getNumVertices(), numVertices)

    def testGetNumFeatures(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)

        self.assertEquals(sGraph.getNumVertexFeatures(), numFeatures)


    def testGetEdge(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        sGraph.addEdge(2, 5, 1)
        sGraph.addEdge(4, 8, 34)

        self.assertEquals(sGraph.getEdge(2, 5), 1)
        self.assertEquals(sGraph.getEdge(5, 2), 1)
        self.assertEquals(sGraph.getEdge(4, 8), 34)
        self.assertEquals(sGraph.getEdge(8, 4), 34)

        self.assertEquals(sGraph.getEdge(4, 4), None)

    def testGetVertex(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        sGraph.setVertex(1, numpy.array([4, 5, 2]))

        self.assertRaises(ValueError, sGraph.setVertex, -1, numpy.array([4, 5, 2]))
        self.assertRaises(ValueError, sGraph.setVertex, 11, numpy.array([4, 5, 2]))
        self.assertRaises(ValueError, sGraph.setVertex, 2, numpy.array([4, 5, 2, 8]))
        self.assertRaises(ValueError, sGraph.setVertex, 2, numpy.array([4, 5]))
        self.assertTrue((sGraph.getVertex(1) == numpy.array([4, 5, 2])).all())
        self.assertTrue((sGraph.getVertex(0) == numpy.array([0, 0, 0])).all())

    def testSetVertex(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)
        sGraph.setVertex(1, numpy.array([4, 5, 2]))

        self.assertTrue((sGraph.getVertex(1) == numpy.array([4, 5, 2])).all())
        self.assertTrue((sGraph.getVertex(0) == numpy.array([0, 0, 0])).all())

        sGraph.setVertex(1, numpy.array([8, 3, 1]))
        self.assertTrue((sGraph.getVertex(1) == numpy.array([8, 3, 1])).all())

    def testIsUndirected(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList, True)
        self.assertEquals(sGraph.isUndirected(), True)

        sGraph = SparseGraph(vList, False)
        self.assertEquals(sGraph.isUndirected(), False)

    def testGetAllVertexIds(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        sGraph = SparseGraph(vList)

        self.assertTrue((sGraph.getAllVertexIds() == numpy.array(range(0, numVertices))).all())

    def testGetSubGraph(self):
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        vertices = numpy.random.rand(numVertices, numFeatures)
        vList.setVertices(vertices)

        sGraph = SparseGraph(vList)
        sGraph.addEdge(0, 1)
        sGraph.addEdge(0, 2)
        sGraph.addEdge(0, 3)
        sGraph.addEdge(2, 1)
        sGraph.addEdge(2, 5)
        sGraph.addEdge(2, 6)
        sGraph.addEdge(6, 9)

        subgraph = sGraph.subgraph([0,1,2,3])

        self.assertEquals(subgraph.getNumVertices(), 4)
        self.assertEquals(subgraph.getNumVertexFeatures(), numFeatures)
        self.assertTrue((subgraph.getVertexList().getVertices(range(0, 4)) == vertices[range(0,4), :]).all())
        self.assertEquals(subgraph.getNumEdges(), 4)
        self.assertTrue(subgraph.getEdge(0, 1) == 1)
        self.assertTrue(subgraph.getEdge(0, 2) == 1)
        self.assertTrue(subgraph.getEdge(0, 3) == 1)
        self.assertTrue(subgraph.getEdge(2, 1) == 1)

        subgraph = sGraph.subgraph([1,2,5,6])
        self.assertEquals(subgraph.getNumVertices(), 4)
        self.assertEquals(subgraph.getNumVertexFeatures(), numFeatures)
        self.assertEquals(subgraph.getNumEdges(), 3)
        self.assertTrue((subgraph.getVertexList().getVertices([0,1,2,3]) == vertices[[1,2,5,6], :]).all())
        self.assertTrue(subgraph.getEdge(0, 1) == 1)
        self.assertTrue(subgraph.getEdge(1, 2) == 1)
        self.assertTrue(subgraph.getEdge(1, 3) == 1)


        #Test case of directed graph
        numVertices = 10
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)
        vertices = numpy.random.rand(numVertices, numFeatures)
        vList.setVertices(vertices)

        sGraph = SparseGraph(vList, False)
        sGraph.addEdge(0, 1)
        sGraph.addEdge(0, 2)
        sGraph.addEdge(0, 3)
        sGraph.addEdge(2, 1)
        sGraph.addEdge(2, 5)
        sGraph.addEdge(2, 6)
        sGraph.addEdge(6, 9)

        subgraph = sGraph.subgraph([0,1,2,3])

        self.assertEquals(subgraph.isUndirected(), False)
        self.assertEquals(subgraph.getNumVertices(), 4)
        self.assertEquals(subgraph.getNumVertexFeatures(), numFeatures)
        self.assertTrue((subgraph.getVertexList().getVertices(range(0, 4)) == vertices[range(0,4), :]).all())
        self.assertEquals(subgraph.getNumEdges(), 4)
        self.assertTrue(subgraph.getEdge(0, 1) == 1)
        self.assertTrue(subgraph.getEdge(0, 2) == 1)
        self.assertTrue(subgraph.getEdge(0, 3) == 1)
        self.assertTrue(subgraph.getEdge(2, 1) == 1)

        subgraph = sGraph.subgraph([1,2,5,6])
        self.assertEquals(subgraph.getNumVertices(), 4)
        self.assertEquals(subgraph.getNumVertexFeatures(), numFeatures)
        self.assertEquals(subgraph.getNumEdges(), 3)
        self.assertTrue((subgraph.getVertexList().getVertices([0,1,2,3]) == vertices[[1,2,5,6], :]).all())
        self.assertTrue(subgraph.getEdge(1, 0) == 1)
        self.assertTrue(subgraph.getEdge(1, 2) == 1)
        self.assertTrue(subgraph.getEdge(1, 3) == 1)
        
    def testEdgeAddition(self):
        numVertices = 5
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        sGraph = SparseGraph(vList, False)
        sGraph.addEdge(0, 1)
        sGraph.addEdge(0, 2)
        sGraph.addEdge(0, 3)
        sGraph.addEdge(2, 1)

        sGraph2 = SparseGraph(vList, False)
        sGraph2.addEdge(3, 2)
        sGraph2.addEdge(0, 4)
        sGraph2.addEdge(1, 3)
        sGraph2.addEdge(2, 1)

        sGraph.edgeAddition(sGraph2)

        self.assertEquals(sGraph.getEdge(0,1) , 1)
        self.assertEquals(sGraph.getEdge(0,2) , 1)
        self.assertEquals(sGraph.getEdge(3,2) , 1)
        self.assertEquals(sGraph.getEdge(2,1) , 2)

        #Test edge addition of different sized graphs
        vList2 = VertexList(numVertices-1, numFeatures)
        sGraph2 = SparseGraph(vList2, False)
        sGraph2.addEdge(3, 2)

        self.assertRaises(ValueError, sGraph.edgeAddition, sGraph2)

    def testGetCopy(self):
        numVertices = 5
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        sGraph = SparseGraph(vList, False)
        sGraph.addEdge(0, 1)
        sGraph.addEdge(0, 2)
        sGraph.addEdge(0, 3)
        sGraph.addEdge(2, 1)

        sGraph2 = sGraph.copy()

        sGraph2.addEdge(3, 4)

        self.assertEquals(sGraph2.getEdge(3, 4), 1)
        self.assertEquals(sGraph.getEdge(3, 4), None)
        
    def testGetEgoGraph(self):
        numVertices = 6
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        sGraph = SparseGraph(vList, True)
        sGraph.addEdge(0, 1)
        sGraph.addEdge(0, 2)
        sGraph.addEdge(0, 3)
        sGraph.addEdge(2, 1)
        sGraph.addEdge(2, 3)
        sGraph.addEdge(4, 1)

        egoGraph = sGraph.egoGraph(0)

        self.assertTrue(egoGraph.getNumVertices() == 4)
        self.assertTrue(egoGraph.getNumEdges() == 5)
        self.assertEquals(egoGraph.getEdge(0,1), 1)
        self.assertEquals(egoGraph.getEdge(0,2), 1)
        self.assertEquals(egoGraph.getEdge(0,3), 1)
        self.assertEquals(egoGraph.getEdge(2,1), 1)
        self.assertEquals(egoGraph.getEdge(2,3), 1)

        egoGraph = sGraph.egoGraph(4)

        self.assertTrue(egoGraph.getNumVertices() == 2)
        self.assertTrue(egoGraph.getNumEdges() == 1)
        self.assertEquals(egoGraph.getEdge(1,0), 1)

        egoGraph = sGraph.egoGraph(3)

        self.assertTrue(egoGraph.getNumVertices() == 3)
        self.assertTrue(egoGraph.getNumEdges() == 3)
        self.assertEquals(egoGraph.getEdge(0,2), 1)
        self.assertEquals(egoGraph.getEdge(0,1), 1)
        self.assertEquals(egoGraph.getEdge(2,1), 1)


        egoGraph = sGraph.egoGraph(5)
        self.assertTrue(egoGraph.getNumVertices() == 1)
        self.assertTrue(egoGraph.getNumEdges() == 0)

    def testStr(self):
        print(self.sGraph)

    def testRemoveAllEdges(self):
        numVertices = 6
        numFeatures = 3
        vList = VertexList(numVertices, numFeatures)

        sGraph = SparseGraph(vList, True)
        sGraph.addEdge(0, 1)
        sGraph.addEdge(0, 2)
        sGraph.addEdge(0, 3)
        sGraph.addEdge(2, 1)
        sGraph.addEdge(2, 3)
        sGraph.addEdge(4, 1)

        sGraph.removeAllEdges()
        self.assertTrue(sGraph.getEdge(0,1) == None)

    def testGetAdjacencyMatrix(self):
        numVertices = 3
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)

        sGraph = SparseGraph(vList, False)
        sGraph.addEdge(0, 1, 0.5)
        sGraph.addEdge(2, 1, 0.2)
        sGraph.addEdge(1, 1, 0.1)

        A = sGraph.adjacencyMatrix()
        W = sGraph.getWeightMatrix()
        

        W2 = numpy.zeros((numVertices, numVertices))
        A2 = numpy.zeros((numVertices, numVertices))

        W2[0,1]= 0.5
        W2[2,1]= 0.2
        W2[1,1]= 0.1

        A2[0,1]= 1
        A2[2,1]= 1
        A2[1,1]= 1

        self.assertTrue((W == W2).all())
        self.assertTrue((A == A2).all())

    def testComplement(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList)

        graph3 = graph.complement()

        self.assertTrue(graph3.isUndirected())
        self.assertEquals(graph3.getNumEdges(), (numVertices**2 + numVertices)/2)

        graph.addEdge(0, 1, 0.1)
        graph.addEdge(2, 1, 0.2)
        graph.addEdge(4, 2, 0.5)
        graph.addEdge(6, 7, 0.9)
        graph.addEdge(3, 3, 1.1)

        graph2 = graph.complement()

        self.assertTrue(graph2.isUndirected())
        self.assertEquals(graph2.getEdge(0, 1), None)
        self.assertEquals(graph2.getEdge(2, 1), None)
        self.assertEquals(graph2.getEdge(4, 2), None)
        self.assertEquals(graph2.getEdge(6, 7), None)
        self.assertEquals(graph2.getEdge(3, 3), None)

        self.assertEquals(graph2.getEdge(0,0), 1)
        self.assertEquals(graph2.getNumEdges(), 50)

        #Now test on directed graphs
        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList, False)

        graph3 = graph.complement()
        self.assertEquals(graph3.getNumEdges(), numVertices**2) 

        graph.addEdge(0, 1, 0.1)
        graph.addEdge(2, 1, 0.2)
        graph.addEdge(4, 2, 0.5)
        graph.addEdge(6, 7, 0.9)
        graph.addEdge(3, 3, 1.1)

        graph2 = graph.complement()

        self.assertFalse(graph2.isUndirected())
        self.assertEquals(graph2.getEdge(0, 1), None)
        self.assertEquals(graph2.getEdge(2, 1), None)
        self.assertEquals(graph2.getEdge(4, 2), None)
        self.assertEquals(graph2.getEdge(6, 7), None)
        self.assertEquals(graph2.getEdge(3, 3), None)

        self.assertEquals(graph2.getEdge(0,0), 1)
        self.assertEquals(graph2.getEdge(1,0), 1)
        self.assertEquals(graph2.getNumEdges(), 95)

    def testFindTrees(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList, False)

        graph.addEdge(0, 1, 1)
        graph.addEdge(0, 2, 1)
        graph.addEdge(1, 3, 1)
        graph.addEdge(4, 5, 1)
        graph.addEdge(6, 7, 1)

        trees = graph.findTrees()

        self.assertEquals(trees[0], set([0,1,2,3]))
        self.assertEquals(trees[1], set([4,5]))
        self.assertEquals(trees[2], set([6,7]))
        self.assertEquals(trees[3], set([8]))
        self.assertEquals(trees[4], set([9]))
        
    def testSetWeightMatrix(self):
        numVertices = 5
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        vList.setVertices(numpy.random.rand(numVertices, numFeatures))
        graph = SparseGraph(vList)

        graph.addEdge(0, 1)
        graph.addEdge(0, 2)

        W = numpy.zeros((numVertices, numVertices))
        W[1, 1] = 1
        W[2, 1] = 1
        W[1, 2] = 1

        graph.setWeightMatrix(W)
        self.assertTrue((graph.getAllEdges() == numpy.array([[1, 1], [2, 1]])).all())

        W[1, 3] = 1
        self.assertRaises(ValueError, graph.setWeightMatrix, W)
        W = numpy.zeros((numVertices, numVertices+1))
        self.assertRaises(ValueError, graph.setWeightMatrix, W)

        #Now, see if it works for undirected graphs
        graph = SparseGraph(vList, False)
        W = numpy.zeros((numVertices, numVertices))
        W[1, 0] = 1
        W[3, 1] = 1
        W[1, 3] = 1

        graph.setWeightMatrix(W)
        self.assertTrue((graph.getAllEdges() == numpy.array([[1, 0], [1,3], [3, 1]])).all())

    def testGetNumDirEdges(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList)
        graph.addEdge(0, 1, 0.1)
        graph.addEdge(1, 2, 0.1)

        self.assertTrue(graph.getNumDirEdges() == 4)
        graph.addEdge(1, 1)
        self.assertTrue(graph.getNumDirEdges() == 5)

        graph = SparseGraph(vList, False)
        graph.addEdge(0, 1)
        graph.addEdge(1, 2)

        self.assertTrue(graph.getNumDirEdges() == 2)
        graph.addEdge(1, 1)
        self.assertTrue(graph.getNumDirEdges() == 3)

    def testOutDegreeSequence(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList)
        graph.addEdge(0, 1, 0.1)
        graph.addEdge(1, 2, 0.2)
        graph.addEdge(1, 5)

        self.assertTrue((graph.outDegreeSequence() == numpy.array([1, 3, 1, 0,0,1,0,0,0,0])).all() )

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList, False)
        graph.addEdge(0, 1)
        graph.addEdge(1, 2)
        graph.addEdge(1, 5)
        graph.addEdge(3, 3)

        self.assertTrue((graph.outDegreeSequence() == numpy.array([1, 2, 0, 1,0,0,0,0,0,0])).all() )

    def testInDegreeSequence(self):
        numVertices = 10
        numFeatures = 1

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList)
        graph.addEdge(0, 1)
        graph.addEdge(1, 2)
        graph.addEdge(1, 5)

        self.assertTrue((graph.inDegreeSequence() == numpy.array([1, 3, 1, 0,0,1,0,0,0,0])).all() )

        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList, False)
        graph.addEdge(0, 1, 0.1)
        graph.addEdge(1, 2, 0.2)
        graph.addEdge(1, 5)
        graph.addEdge(2, 1)
        graph.addEdge(3, 3)

        self.assertTrue((graph.inDegreeSequence() == numpy.array([0, 2, 1, 1,0,1,0,0,0,0])).all() )

    def testGeneralVertexList(self):
        #Very brief test to make sure sparse graph works with general vertex lists
        numVertices = 10
        vList = GeneralVertexList(numVertices)
        vList.setVertex(0, "a")
        vList.setVertex(1, "b")
        vList.setVertex(5, "c")

        graph = SparseGraph(vList)
        graph.addEdge(0, 1)
        graph.addEdge(1, 2)
        graph.addEdge(1, 5)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()