

import numpy
import numpy.random as rand


class Latex(object):
    '''
    A class with some general useful functions for Latex output.
    '''
    def __init__(self):
        pass

    @staticmethod
    def array2DToLatex(X):
        """
        Take a 2D array and print a Latex table
        """
        if X.ndim != 2:
            raise ValueError("Array must be one dimensional")

        n = X.shape[0]
        outputStr = ""

        for i in range(n):
            outputStr += Latex.array1DToLatex(X[i, :]) + "\\\\\n"

        return outputStr

    @staticmethod
    def array1DToLatex(X):
        """
        Take a 1D numpy array and print in latex table format x1 & x2 .. xn
        """
        if X.ndim != 1:
            raise ValueError("Array must be one dimensional")

        n = X.shape[0]
        outputStr = ""

        for i in range(0, n):
            if i != n-1:
                outputStr += "%.3f & " % X[i]
            else:
                outputStr += "%.3f" % X[i]

        return outputStr


    @staticmethod
    def meanStdArraysToLatex(means, stds):
        """
        A method which will print one line of a latex table using a 1D array of means
        and standard deviations. Prints in format mean1 (std1) & mean2 (std2) ...
        """
        if means.ndim != 1 or stds.ndim != 1:
            raise ValueError("Array must be one dimensional")
        if means.shape != stds.shape:
            raise ValueError("Arrays must be the same shape")

        n = means.shape[0]
        outputStr = ""

        for i in range(0, n):
            if i != n-1:
                outputStr += "%.3f(%.3f) & " % (means[i], stds[i])
            else:
                outputStr += "%.3f(%.3f)" % (means[i], stds[i])

        return outputStr