'''
Created on 31 Jul 2009

@author: charanpal
'''
import unittest
import numpy
import logging
import sys
from apgl.util.Util import Util
from apgl.util.PathDefaults import PathDefaults
from apgl.io.EgoCsvReader import EgoCsvReader

#TODO: Test sampleWithoutReplacemnt 
#TODO: Test randNormalInt 

class UtilTest(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)


    def tearDown(self):
        pass


    def testHistogram(self):
        v = numpy.array([0, 0, 1, 5, 0, 2, 2, 2, 5])
        
        (freq, items) = Util.histogram(v)
        self.assertTrue((freq == numpy.array([3, 1, 3, 2])).all())
        self.assertTrue((items == numpy.array([0, 1, 2, 5])).all())

    def testComputeMeanVar(self):
        eCsvReader = EgoCsvReader()
        fileName = PathDefaults.getDataDir()  +  "infoDiffusion/EgoData.csv"
        questionIds = eCsvReader.getEgoQuestionIds()
        (X, _) = eCsvReader.readFile(fileName, questionIds)

        (means, vars) = Util.computeMeanVar(X)

        means2 = sum(X, 0)/X.shape[0]
        X2 = X - means2
        vars2 = numpy.dot(X2.T,X2)/X.shape[0]

        self.assertTrue((means2 == means).all())
        self.assertAlmostEquals(numpy.linalg.norm(vars2 - vars), 0)

        print(numpy.linalg.norm(vars.T - vars))
        self.assertTrue(numpy.linalg.norm(vars.T - vars) < 10**-5)

        

    def testMode(self):
        x = numpy.array([1,1,1,2,2,3,3,3,3,3,5,5])
        self.assertEquals(Util.mode(x), 3)

        x = numpy.array([1,1,1,2,2,3,3,3,5,5])
        self.assertEquals(Util.mode(x), 1)

        x = numpy.array([1,2,3,4])
        self.assertEquals(Util.mode(x), 1)

        x = numpy.array([0])
        self.assertEquals(Util.mode(x), 0)

    def testCountFrequencies(self):
        itemArray = numpy.array([1, 2, 2, 5, 12, 1, 8])
        numElements = 13

        freqs = Util.countFrequencies(itemArray, numElements)

        self.assertTrue((freqs == numpy.array([0, 2, 2, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1 ])).all())

        itemArray = numpy.array([1, 2])
        numElements = 3
        freqs = Util.countFrequencies(itemArray, numElements)

        self.assertTrue((freqs == numpy.array([0, 1, 1])).all())

    def testMdot(self):
        A = numpy.random.rand(7, 2)
        B = numpy.random.rand(2, 3)
        C = numpy.random.rand(3, 8)

        tol = 0.01

        X = Util.mdot(A, B)
        X2 = numpy.dot(A, B)
        self.assertTrue((X == X2).all())

        logging.debug("=============")
        X = Util.mdot(A, B, C)
        X2 = numpy.dot(A, numpy.dot(B, C))
        self.assertTrue(numpy.linalg.norm(X - X2) < tol)

        A = numpy.random.rand(7, 2)
        B = numpy.random.rand(2, 10)
        C = numpy.random.rand(10, 8)

        logging.debug("=============")
        X = Util.mdot(A, B, C)
        X2 = numpy.dot(A, numpy.dot(B, C))
        self.assertTrue(numpy.linalg.norm(X - X2) < tol)

        A = numpy.random.rand(7, 2)
        B = numpy.random.rand(2, 10)
        C = numpy.random.rand(10, 8)
        D = numpy.random.rand(8, 1)

        logging.debug("=============")
        X = Util.mdot(A, B, C, D)
        X2 = numpy.dot(numpy.dot(A, numpy.dot(B, C)), D)
        self.assertTrue(numpy.linalg.norm(X - X2) < tol)

        #Test with vectors
        a = numpy.random.rand(10)
        B = numpy.random.rand(10, 5)
        c = numpy.random.rand(5)

        logging.debug("=============")
        X = Util.mdot(a.T, B, c)
        X2 = numpy.dot(a.T, numpy.dot(B, c))
        self.assertTrue(numpy.linalg.norm(X - X2) < tol)

        #Test outer products
        logging.debug("==============")
        X = Util.mdot(a.T, a, c.T, c)
        X2 = numpy.dot(a, numpy.dot(a.T, numpy.dot(c.T, c)))
        self.assertTrue(numpy.linalg.norm(X - X2) < tol)

        A = numpy.random.rand(10, 10)

        logging.debug("==============")
        X = Util.mdot(a.T, A, a)
        X2 = numpy.dot(a, numpy.dot(A.T, a))
        self.assertTrue(numpy.linalg.norm(X - X2) < tol)

    def testRank(self):
        X = numpy.random.rand(10, 1)
        self.assertEquals(Util.rank(X), 1)

        X = numpy.random.rand(10, 12)
        self.assertEquals(Util.rank(X), 10)

        X = numpy.random.rand(31, 12)
        self.assertEquals(Util.rank(X), 12)

        K = numpy.dot(X, X.T)
        self.assertEquals(Util.rank(X), 12)

    def testPrintIteration(self):
        Util.printIteration(0, 1, 1)
        Util.printIteration(0, 1, 10)
        Util.printIteration(9, 1, 10)
        Util.printIteration(9, 1, 11)
        Util.printIteration(1, 1, 7)


    def testRandomChoice(self):
        v = numpy.array([0.25, 0.25, 0.25])

        tol = 10**-2
        c = numpy.zeros(3)
        numSamples = 500

        for i in range(numSamples):
            j = Util.randomChoice(v)
            print(j)
            c[j] += 1

        self.assertTrue((c/numSamples == numpy.array([0.33, 0.33, 0.33])).all() < tol)

        v = v * 20
        c = numpy.zeros(3)

        for i in range(numSamples):
            j = Util.randomChoice(v)
            print(j)
            c[j] += 1

        self.assertTrue((c/numSamples == numpy.array([0.33, 0.33, 0.33])).all() < tol)

        #Now try different distribution 
        v = numpy.array([0.2, 0.6, 0.2])

        c = numpy.zeros(3)

        for i in range(numSamples):
            j = Util.randomChoice(v)
            print(j)
            c[j] += 1

        self.assertTrue((c/numSamples == v).all() < tol)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    
    unittest.main()