import numpy
from apgl.graph import *
from apgl.util import * 
from apgl.viroscopy.HIVIndividual import HIVIndividual
import matplotlib.pyplot as pyplot 

"""
This is the epidemic model modified from Chi's orignal description. 
"""

#Rates for various events (probably should include time as parameter too)
def infectionRate(vertex1, vertex2):
    heterorate = 0.5
    birate = 0.5

    if vertex1.getFreeDegree() == 0 or vertex2.getFreeDegree() == 0:
        return 0.0

    if vertex1.getGender() != vertex2.getGender():
        return heterorate
    elif vertex1.getGender() == vertex2.getGender() and vertex1.getOrientation()==1 and vertex1.getOrientation()==1:
        return birate
    else:
        return 0.0

def randomDetectionRate(vertex):
    return 0.5

def contactTracingRate(vertex):
    return 0.5

#Total number of people in population
#All vertices start as suseptible
t = 0  #initial time
M = 1000
vList = GeneralVertexList(M)

for i in range(M):
    vertex = HIVIndividual()
    vList.setVertex(i, vertex)

#Graph is one in which edges represent an infection 
undirected = False 
graph = SparseGraph(vList, undirected)

#Choose a set of people to be infected
numInitialInfected = 20
defaultFreeDegree = 5

susceptibleSet = set([])
infectedSet = set([])
removedSet = set([])

for i in range(numInitialInfected):
    vList.getVertex(i).setFreeDegree(defaultFreeDegree)
    vList.getVertex(i).setInfected(0.0)
    infectedSet.add(i)

    print(vList.getVertex(i))

for i in range(numInitialInfected, M):
    vList.getVertex(i).setFreeDegree(defaultFreeDegree)
    susceptibleSet.add(i)

#End time 
T = 1
iter = 0

#Sizes of the sets
susceptibleSizes = []
infectedSizes = []
removedSizes = []

times = [] 

#Now, start the loop
while t < T and len(infectedSet) != 0:
    print("t=" + str(t) + " S=" + str(len(susceptibleSet)) + " I=" + str(len(infectedSet)) + " R=" + str(len(removedSet)))
    #First lets work out the time of the next event
    ind = 0 
    infectRates = numpy.zeros((len(infectedSet)*len(susceptibleSet), 3))

    #Compute infect rates 
    for i in infectedSet:
        for j in susceptibleSet:
            infected = vList.getVertex(i)
            suscesptible = vList.getVertex(j)
            infectRates[ind, :] = numpy.array([i, j, infectionRate(infected, suscesptible)])
            ind = ind + 1 

    lmbdat = numpy.sum(infectRates[:, 2]) 

    #Compute detection rates
    randomDetectRates = numpy.zeros((len(infectedSet), 2))
    contactTracingRates = numpy.zeros((len(infectedSet), 2))

    ind = 0 
    for i in infectedSet:
        infected = vList.getVertex(i)
        randomDetectRates[ind] = numpy.array([i, randomDetectionRate(infected)]) 
        contactTracingRates[ind] = numpy.array([i, contactTracingRate(infected)]) 
        ind += 1 

    muRSt = numpy.sum(randomDetectRates)
    muCTt = numpy.sum(contactTracingRates)

    #print(lmbdat)
    #print(muRSt)
    #print(muCTt)

    rhot = lmbdat + muRSt + muCTt

    #Now generate random variable which is the advancement in time 
    tauPrime = numpy.random.exponential(1/rhot)
    t = t + tauPrime 

    #Now compute the probabilities of each event type 
    infectionProb = lmbdat/rhot
    detectionRandom = muRSt/rhot
    detectionContact = muCTt/rhot

    #Compute random variable
    p = numpy.random.rand()

    if p < infectionProb:
        eventInd = Util.randomChoice(infectRates[:, 2])
        newInfectedIndex = infectRates[eventInd, 1]
        newInfected = vList.getVertex(newInfectedIndex)
        newInfected.setInfected(t)

        graph.addEdge(infectRates[eventInd, 0], infectRates[eventInd, 1])
        infectedSet.add(newInfectedIndex)
        susceptibleSet.remove(newInfectedIndex)
    elif p >= infectionProb and p < infectionProb+detectionRandom:
        eventInd = Util.randomChoice(randomDetectRates)
        newDetectedIndex = randomDetectRates[eventInd, 0]
        newDetected = vList.getVertex(newDetectedIndex)
        newDetected.setDetected(t)

        removedSet.add(newDetectedIndex)
        infectedSet.remove(newDetectedIndex)
    else:
        eventInd = Util.randomChoice(contactTracingRates)
        newDetectedIndex = contactTracingRates[eventInd, 0]
        newDetected = vList.getVertex(newDetectedIndex)
        newDetected.setDetected(t)

        removedSet.add(newDetectedIndex)
        infectedSet.remove(newDetectedIndex)

    #Items to record at each iteration 
    susceptibleSizes.append(len(susceptibleSet))
    infectedSizes.append(len(infectedSet))
    removedSizes.append(len(removedSet))

    times.append(t)

    iter += 1


print("Total time passed: " + str(t))
print("Susceptible Set size: " + str(len(susceptibleSet)))
print("Infected Set size: " + str(len(infectedSet)))
print("Removed Set size: " + str(len(removedSet)))

times = numpy.array(times)
susceptibleSizes = numpy.array(susceptibleSizes)
infectedSizes = numpy.array(infectedSizes)
removedSizes = numpy.array(removedSizes)

pyplot.plot(times, infectedSizes, 'b-', times, removedSizes, 'g--')
#pyplot.plot(times, susceptibleSizes, 'r+', times, infectedSizes, 'b-', times, removedSizes, 'g--')
pyplot.show()

#TODO: program in delay in detection using CT
#Generate complete outputs -diameter, C
#Optimise 