from apgl.viroscopy.HIVGraphReader import HIVGraphReader
import logging
import sys
import matplotlib
import numpy 
matplotlib.use('WXAgg') # do this before importing pylab
import matplotlib.pyplot as plt
from apgl.util import * 


"""
This script computes some basic statistics on the growing graph. We currently
combine both infection and detection graphs (which are both directed). 
"""


logging.basicConfig(stream=sys.stdout, level=logging.INFO)

hivReader = HIVGraphReader()
graph = hivReader.readHIVGraph()
fInds = hivReader.getFeatureIndices()

figureDir = PathDefaults.getOutputDir() + "viroscopy/figures/"

#The set of edges indexed by zeros is the contact graph
#The ones indexed by 1 is the infection graph
edgeTypeIndex1 = 0
edgeTypeIndex2 = 1
sGraphContact = graph.getSparseGraph(edgeTypeIndex1)
sGraphInfect = graph.getSparseGraph(edgeTypeIndex2)
sGraphContact.edgeAddition(sGraphInfect)

sGraph = sGraphContact

logging.info(sGraph)
logging.info("Number of vertices: " + str(sGraph.getNumVertices()))
logging.info("Number of features: " + str(sGraph.getNumVertexFeatures()))
logging.info("Number of edges: " + str(sGraph.getNumEdges()))
logging.info("Computing all components")

componentIndex = 0
components = sGraph.findConnectedComponents()
logging.info("Found all components")
logging.info("Largest component is " + str(len(components[0])))
logging.info("Number of components " + str(len(components)))

#sGraph = sGraph.subgraph(components[componentIndex])
vertexArray = sGraph.getVertexList().getVertices(range(0, sGraph.getVertexList().getNumVertices()))

logging.info("Size of graph we will use: " + str(sGraph.getNumVertices()))

#Some indices
dobIndex = fInds["birthDate"]
detectionIndex = fInds["detectDate"]
deathIndex = fInds["deathDate"]
genderIndex = fInds["gender"]
orientationIndex = fInds["orient"]

meanAgeAtDetection = numpy.mean(vertexArray[:, detectionIndex] - vertexArray[:, dobIndex])
stdAgeAtDetection = numpy.std(vertexArray[:, detectionIndex] - vertexArray[:, dobIndex])
meanDeathTime = numpy.mean(vertexArray[:, deathIndex] - vertexArray[:, detectionIndex])
stdDeathTime = numpy.std(vertexArray[:, deathIndex] - vertexArray[:, detectionIndex])

startDate = 1900
daysInYear = 365
print("Mean age at detection: " + str(meanAgeAtDetection/daysInYear))
print("Std age at detection: " + str(stdAgeAtDetection/daysInYear))
print(meanDeathTime/daysInYear)
print(stdDeathTime/daysInYear)

q = 0.9

#Measure the evolution of the largest component
maxComponentSizes = []
numComponentsList = []
meanComponentSizes = [] 
diameters = [] #Of current largest component
effectiveDiameters = [] 

#Properties of graph as whole
numEdges = []
minDegrees = []
maxDegrees = []
meanDegrees = []
numVertices = []
densities = []


detectionAges = []
deathAfterInfectAges = []
deathAges = []
femaleMeans = []
homoMeans = []

detections = vertexArray[:, detectionIndex]
timeList = range(int(min(detections)), int(max(detections)), 30)

for i in timeList:
    print("Date: " + str(StringUtils.getDateStrFromDay(i, startDate)))
    subGraphIndices = numpy.nonzero(detections <= i)[0]
    subGraph = sGraph.subgraph(subGraphIndices)

    subComponents = subGraph.findConnectedComponents()
    maxComponentSize = len(subComponents[0])
    numComponents = len(subComponents)
    meanComponentSize = sum([len(x) for x in subComponents])/float(numComponents)
    maxComponentGraph = subGraph.subgraph(subComponents[0])

    degreeDist = subGraph.degreeDistribution()
    meanDegree = float(subGraph.getNumEdges())/subGraph.getNumVertices()
    #diameter = maxComponentGraph.diameter()
    #effectiveDiameter = maxComponentGraph.effectiveDiameter(q)
    diameter = 0
    effectiveDiameter = 0

    minDegrees.append(numpy.min(degreeDist))
    maxDegrees.append(degreeDist.shape[0])
    maxComponentSizes.append(maxComponentSize)
    numComponentsList.append(numComponents)
    meanComponentSizes.append(meanComponentSize)
    numEdges.append(subGraph.getNumEdges())
    numVertices.append(subGraph.getNumVertices())
    meanDegrees.append(meanDegree)
    densities.append(subGraph.density())
    diameters.append(diameter)
    effectiveDiameters.append(effectiveDiameter)

    #Values based on the properties of the vertices
    subgraphVertexArray = subGraph.getVertexList().getVertices(range(0, subGraph.getNumVertices()))
    detectionAges.append(numpy.mean((subgraphVertexArray[:, detectionIndex] - subgraphVertexArray[:, dobIndex]))/daysInYear)
    deathAfterInfectAges.append((numpy.mean(subgraphVertexArray[:, deathIndex] - subgraphVertexArray[:, detectionIndex]))/daysInYear)
    deathAges.append(numpy.mean((subgraphVertexArray[:, deathIndex] - subgraphVertexArray[:, dobIndex]))/daysInYear)

    femaleMeans.append(numpy.mean(subgraphVertexArray[:, genderIndex]))
    homoMeans.append(numpy.mean(subgraphVertexArray[:, orientationIndex]))
    

i = 1

monthList = [float(j)/30 for j in timeList]
print(timeList)
print(monthList)

plt.figure(i)
plt.plot(monthList, maxComponentSizes)
plt.xlabel("Time (months)")
plt.ylabel("Max Component Size")
i += 1

plt.figure(i)
plt.plot(monthList, numComponentsList)
plt.xlabel("Time (months)")
plt.ylabel("Number of components")
plt.savefig(figureDir + "NumComponentsGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, meanComponentSizes)
plt.xlabel("Time (months)")
plt.ylabel("Mean Component Sizes")
plt.savefig(figureDir + "MeanComponentSizeGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, diameters)
plt.xlabel("Time (months)")
plt.ylabel("Max Component Diameter")
plt.savefig(figureDir + "MaxComponentDiameterGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, effectiveDiameters)
plt.xlabel("Time (months)")
plt.ylabel("Max Component Effective Diameter")
plt.savefig(figureDir + "MaxComponentEffDiameterGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, maxDegrees)
plt.xlabel("Time (months)")
plt.ylabel("maxDegrees")
i += 1

plt.figure(i)
plt.plot(monthList, numEdges)
plt.xlabel("Time (months)")
plt.ylabel("numEdges")
i += 1

plt.figure(i)
plt.plot(monthList, numVertices)
plt.xlabel("Time (months)")
plt.ylabel("numVertices")
i += 1

plt.figure(i)
plt.plot(numpy.log(numVertices), numpy.log(numEdges))
plt.xlabel("log(numVertices)")
plt.ylabel("log(numEdges)")
plt.savefig(figureDir + "LogVerticesEdgesGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, meanDegrees)
plt.xlabel("Time (months)")
plt.ylabel("meanDegrees")
i += 1


plt.figure(i)
plt.plot(monthList, densities)
plt.xlabel("Time (months)")
plt.ylabel("densities")
plt.savefig(figureDir + "DensityGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, detectionAges)
plt.xlabel("Time (months)")
plt.ylabel("detectionAges")
plt.savefig(figureDir + "DetectionMeansGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, femaleMeans)
plt.xlabel("Time (months)")
plt.ylabel("femaleMeans")
plt.savefig(figureDir + "FemaleMeansGrowth.eps")
i += 1

plt.figure(i)
plt.plot(monthList, homoMeans)
plt.xlabel("Time (months)")
plt.ylabel("homoMeans")
i += 1

#Note that death has a lot of missing values
"""
plt.figure(i)
plt.plot(timeList, deathAfterInfectAges)
plt.xlabel("Time (months)")
plt.ylabel("deathAfterInfectAges")
i += 1

plt.figure(i)
plt.plot(timeList, deathAges)
plt.xlabel("Time (months)")
plt.ylabel("deathAges")
i += 1
"""

#Now we compute degree distributions, etc. at various time points.
#TODO:
timeList = range(int(min(detections)), int(max(detections)), 730)
monthList = [float(j)/30 for j in timeList]
 
for j in timeList:
    print("Date: " + str(StringUtils.getDateStrFromDay(j, startDate)))
    subGraphIndices = numpy.nonzero(detections <= j)[0]
    subGraph = sGraph.subgraph(subGraphIndices)
    numVertices = float(subGraph.getNumVertices())

    degreeDistribution = subGraph.degreeDistribution()
    W = subGraph.getWeightMatrix()
    #Make sure weight matrix is symmetric 
    W = (W + W.T)/2
    eigenDistribution, V = numpy.linalg.eig(W)
    #Note we just take the magnitude of the eigenvalues 
    eigenDistribution = numpy.sort(eigenDistribution)

    plt.figure(i)
    plt.suptitle("Date: " + str(StringUtils.getDateStrFromDay(j, startDate)))
    plt.plot(numpy.arange(degreeDistribution.shape[0]), degreeDistribution/numVertices)
    plt.xlabel("Degree")
    plt.ylabel("Probability")
    plt.savefig(figureDir + "DegreeDist" + str(j) + ".eps")
    i += 1

    plt.figure(i)
    plt.suptitle("Date: " + str(StringUtils.getDateStrFromDay(j, startDate)))
    plt.plot(numpy.arange(eigenDistribution.shape[0]), eigenDistribution)
    plt.xlabel("Eigenvalue rank")
    plt.ylabel("Eigenvalue")
    plt.savefig(figureDir + "EigenDist" + str(j) + ".eps")
    i += 1

plt.show()


"""
Also clustering (4 clusters), detection type
Probability of adding node based on degree - try to find how we can generate data.
Mean Time between first and last infection for each person 
Degree distribution at various points, eigenvalue at certain points 
"""